var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import { r as registerInstance, c as createEvent, h } from './core-9a645b44.js';
var isCurrentMonth = function (day, year, month) { return day.month.month === month && day.month.year === year; };
var isWeekend = function (day) { return day.dayInWeek === 6 || day.dayInWeek === 0; };
var isToday = function (day, today) { return day.dayInMonth === today.getDate() &&
    day.month.month === today.getMonth() + 1 &&
    day.month.year === today.getFullYear(); };
var isSelected = function (weekDay, _a) {
    var day = _a.day, year = _a.year, month = _a.month;
    return day === weekDay.dayInMonth &&
        month === weekDay.month.month &&
        year === weekDay.month.year;
};
var selectedDayToCalendarDay = function (selectedDay) {
    if (!selectedDay)
        return;
    var _a = selectedDay
        .split('-')
        .map(function (piece) { return parseInt(piece, 10); }), day = _a[0], month = _a[1], year = _a[2];
    return { day: day, month: month, year: year };
};
var dayClass = function (_a) {
    var weekDay = _a.weekDay, month = _a.month, year = _a.year, selectedDay = _a.selectedDay, rangeStart = _a.rangeStart, rangeEnd = _a.rangeEnd, hoverDay = _a.hoverDay;
    var classes = ['day'];
    if (isWeekend(weekDay)) {
        classes.push('weekend');
    }
    if (isToday(weekDay, new Date())) {
        classes.push('today');
    }
    classes.push(isCurrentMonth(weekDay, year, month) ? 'current-month' : 'other-month');
    if (selectedDay && isSelected(weekDay, selectedDay)) {
        classes.push('selected');
    }
    if (rangeStart && (hoverDay || rangeEnd)) {
        var thisDayTs = Date.UTC(weekDay.month.year, weekDay.month.month - 1, weekDay.dayInMonth);
        var rangeStartTs = Date.UTC(rangeStart.year, rangeStart.month - 1, rangeStart.day);
        var hoverOrRangeEndTs = rangeEnd
            ? Date.UTC(rangeEnd.year, rangeEnd.month - 1, rangeEnd.day)
            : hoverDay
                ? Date.UTC(hoverDay.month.year, hoverDay.month.month - 1, hoverDay.dayInMonth)
                : undefined;
        if (rangeStartTs &&
            hoverOrRangeEndTs &&
            ((thisDayTs >= rangeStartTs && thisDayTs <= hoverOrRangeEndTs) ||
                (thisDayTs <= rangeStartTs && thisDayTs >= hoverOrRangeEndTs))) {
            classes.push('in-range');
        }
    }
    return classes.join(' ');
};
var dayNames = function (startOfTheWeek, locale) {
    if (locale === void 0) { locale = 'en-US'; }
    var days = __spreadArrays(Array(7).keys()).map(function (d) { return new Date(2017, 9, d + 1) // must not use UTC here
        .toLocaleString(locale, { weekday: 'long' })
        .slice(0, 2); } // TODO: think of exposing this
    );
    for (var i = 6; i > 6 - startOfTheWeek; i--) {
        var day = days.shift();
        if (day) {
            days.push(day);
        }
    }
    return days;
};
var monthName = function (year, month, locale) {
    if (locale === void 0) { locale = 'en-US'; }
    return new Date(year, month - 1).toLocaleString(locale, { month: 'long' });
}; // must not use UTC here
var range = function (n) { return Array.from(Array(n).keys()); };
var isLeap = function (year) { return year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0); };
var getDaysInMonth = function (year, month) {
    var daysInMonth = [31, -1, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    if (month === 2 /* February */) {
        return isLeap(year) ? 29 : 28;
    }
    return daysInMonth[month - 1];
};
var getPreviousMonth = function (year, month) {
    if (month === 1 /* January */) {
        return {
            month: 12 /* December */,
            year: year - 1
        };
    }
    return {
        month: month - 1,
        year: year
    };
};
var getNextMonth = function (year, month) {
    if (month === 12 /* December */) {
        return {
            month: 1,
            year: year + 1
        };
    }
    return {
        month: month + 1,
        year: year
    };
};
var getPreviousDay = function (year, month, day) {
    if (day === 1) {
        var prevMonth = getPreviousMonth(year, month);
        var lastDayInPrevMonth = getDaysInMonth(prevMonth.year, prevMonth.month);
        var previousDayFromPreviousMonth = new Date(Date.UTC(prevMonth.year, prevMonth.month - 1, lastDayInPrevMonth));
        return {
            dayInMonth: previousDayFromPreviousMonth.getUTCDate(),
            dayInWeek: previousDayFromPreviousMonth.getUTCDay(),
            month: prevMonth
        };
    }
    var previousDay = new Date(Date.UTC(year, month - 1, day - 1));
    return {
        dayInMonth: previousDay.getUTCDate(),
        dayInWeek: previousDay.getUTCDay(),
        month: { month: month, year: year }
    };
};
var getNextDay = function (year, month, day) {
    var isLastDayInMonth = function (y, m, d) { return getDaysInMonth(y, m) === d; };
    if (isLastDayInMonth(year, month, day)) {
        var nextMonth = getNextMonth(year, month);
        var nextDayOfNextMonth = new Date(Date.UTC(nextMonth.year, nextMonth.month - 1, 1));
        return {
            dayInMonth: nextDayOfNextMonth.getUTCDate(),
            dayInWeek: nextDayOfNextMonth.getUTCDay(),
            month: nextMonth
        };
    }
    var nextDay = new Date(Date.UTC(year, month - 1, day + 1));
    return {
        dayInMonth: nextDay.getUTCDate(),
        dayInWeek: nextDay.getUTCDay(),
        month: { month: month, year: year }
    };
};
/**
 * Get calendar for given month.
 *
 * @param {number} year - year [1900 – 2100]
 * @param {number} month - Month [1  - 12]
 * @param {number} [startOfTheWeek=0] - Start of the week [0 – 6] where 0 is Sunday, and 6 is Saturday
 *
 * @returns {Object[][]}
 */
function calendarMonth(year, month, startOfTheWeek /* Sunday */) {
    if (startOfTheWeek === void 0) { startOfTheWeek = 0; }
    year = parseInt(year, 10);
    month = parseInt(month, 10);
    startOfTheWeek = parseInt(startOfTheWeek, 10);
    if (!year || isNaN(year) || year < 1900 || year > 2100) {
        throw Error('Year can be a number between 1900 and 3000');
    }
    if (!month || isNaN(month) || month < 1 || month > 12) {
        throw Error('Month can be a number from 1 and 12');
    }
    if ((!startOfTheWeek && startOfTheWeek !== 0) || isNaN(startOfTheWeek) || startOfTheWeek < 0 || startOfTheWeek > 6) {
        throw Error('Start of the week can be a number from 0 and 6');
    }
    var firstDayInMonth = new Date(Date.UTC(year, month - 1, 1));
    var startingDay = {
        dayInMonth: firstDayInMonth.getUTCDate(),
        dayInWeek: firstDayInMonth.getUTCDay(),
        month: { month: month, year: year }
    };
    // go back to first start of the week (Monday or Sunday, or any other really)
    if (startingDay.dayInWeek !== startOfTheWeek) {
        var previousDay = getPreviousDay(startingDay.month.year, startingDay.month.month, startingDay.dayInMonth);
        while (previousDay.dayInWeek !== startOfTheWeek) {
            previousDay = getPreviousDay(previousDay.month.year, previousDay.month.month, previousDay.dayInMonth);
        }
        startingDay = previousDay;
    }
    var DAYS_IN_WEEK = 7;
    var WEEKS_IN_CALENDAR = 6;
    var MONTH_CALENDAR_DAYS_NUMBER = DAYS_IN_WEEK * WEEKS_IN_CALENDAR;
    var days = [];
    var currentDay = startingDay;
    // then go next day up until all 42 (6 weeks) are filled
    while (days.length < MONTH_CALENDAR_DAYS_NUMBER) {
        days.push(currentDay);
        currentDay = getNextDay(currentDay.month.year, currentDay.month.month, currentDay.dayInMonth);
    }
    return [
        days.slice(DAYS_IN_WEEK * 0, DAYS_IN_WEEK * 1),
        days.slice(DAYS_IN_WEEK * 1, DAYS_IN_WEEK * 2),
        days.slice(DAYS_IN_WEEK * 2, DAYS_IN_WEEK * 3),
        days.slice(DAYS_IN_WEEK * 3, DAYS_IN_WEEK * 4),
        days.slice(DAYS_IN_WEEK * 4, DAYS_IN_WEEK * 5),
        days.slice(DAYS_IN_WEEK * 5, DAYS_IN_WEEK * 6)
    ];
}
var chromeBordersFix = function (table) {
    table.style.borderSpacing = table.style.borderSpacing === '0px' ? '' : '0px';
};
var CalyCalendar = /** @class */ (function () {
    function CalyCalendar(hostRef) {
        registerInstance(this, hostRef);
        this.tables = [];
        /** (optional) Locale */
        this.locale = 'en-US';
        /** (optional) Start of the week. 0 for Sunday, 1 for Monday, etc. */
        this.startOfTheWeek = 0;
        /** (optional) Number of months rendered */
        this.numberOfMonths = 1;
        /** (optional) Range */
        this.range = false;
        this.daySelected = createEvent(this, "daySelected", 7);
        this.rangeStartSelected = createEvent(this, "rangeStartSelected", 7);
        this.rangeEndSelected = createEvent(this, "rangeEndSelected", 7);
    }
    CalyCalendar.prototype.handleDayClick = function (day) {
        var dayInMonth = day.dayInMonth.toString().padStart(2, '0');
        var month = day.month.month.toString().padStart(2, '0');
        var selectedDay = dayInMonth + "-" + month + "-" + day.month.year;
        if (this.range) {
            if (!this.rangeStart) {
                this.rangeStart = selectedDay;
                this.rangeStartSelected.emit(selectedDay);
            }
            else if (!this.rangeEnd) {
                this.rangeEnd = selectedDay;
                this.rangeEndSelected.emit(selectedDay);
            }
            else {
                this.rangeStart = selectedDay;
                this.rangeEnd = null;
                this.rangeStartSelected.emit(selectedDay);
            }
        }
        else {
            this.selected = selectedDay;
            this.daySelected.emit(selectedDay);
        }
    };
    CalyCalendar.prototype.handleMouseOver = function (day) {
        if (this.range) {
            this.hoverDay = day;
        }
    };
    CalyCalendar.prototype.back = function () {
        var _a = getPreviousMonth(this.year, this.month), month = _a.month, year = _a.year;
        this.month = month;
        this.year = year;
        this.tables.forEach(function (table) { return chromeBordersFix(table); });
    };
    CalyCalendar.prototype.forward = function () {
        var _a = getNextMonth(this.year, this.month), month = _a.month, year = _a.year;
        this.month = month;
        this.year = year;
        this.tables.forEach(function (table) { return chromeBordersFix(table); });
    };
    CalyCalendar.prototype.render = function () {
        var _this = this;
        var month = { month: this.month, year: this.year };
        var months = [];
        for (var _a = 0, _b = range(this.numberOfMonths); _a < _b.length; _a++) {
            var _i = _b[_a];
            months.push({
                year: month.year,
                month: month.month,
                weeks: calendarMonth(month.year, month.month, this.startOfTheWeek),
            });
            month = getNextMonth(month.year, month.month); // mutates month variable to progress it into next month
        }
        return (h("div", { class: "grid" }, h("section", { class: "navigation" }, h("div", { onClick: function () { return _this.back(); }, class: "button" }, h("slot", { name: "back" }, "<")), h("div", { onClick: function () { return _this.forward(); }, class: "button" }, h("slot", { name: "forward" }, ">"))), months.map(function (month) { return (h("div", { class: "inline-flex" }, h("div", { class: "flex justify-center" }, monthName(month.year, month.month, _this.locale), " ", month.year), h("table", { ref: function (el) { return (_this.tables.includes(el) ? {} : _this.tables.push(el)); } }, h("tr", null, dayNames(_this.startOfTheWeek, _this.locale).map(function (dayName) { return (h("td", { class: "borderless day-name" }, dayName)); })), month.weeks.map(function (week) { return (h("tr", null, week.map(function (day) { return (h("td", { class: dayClass({
                weekDay: day,
                month: month.month,
                year: month.year,
                selectedDay: selectedDayToCalendarDay(_this.selected),
                rangeStart: selectedDayToCalendarDay(_this.rangeStart),
                rangeEnd: selectedDayToCalendarDay(_this.rangeEnd),
                hoverDay: _this.hoverDay,
            }), onClick: function () { return _this.handleDayClick(day); }, onMouseOver: function () { return _this.handleMouseOver(day); } }, day.dayInMonth)); }))); })))); })));
    };
    Object.defineProperty(CalyCalendar, "style", {
        get: function () { return ":host{font-family:var(--font,-apple-system,BlinkMacSystemFont);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.grid{display:inline-grid;grid-template-areas:var(--grid,\"nav nav\" \"mn  mn\");-webkit-column-gap:var(--grid-column-gap,12px);-moz-column-gap:var(--grid-column-gap,12px);column-gap:var(--grid-column-gap,12px)}.navigation{grid-area:nav;height:var(--navigation-height,16px)}.inline-flex{display:-ms-inline-flexbox;display:inline-flex;-ms-flex-direction:column;flex-direction:column}.flex,section{display:-ms-flexbox;display:flex}section{-ms-flex-pack:justify;justify-content:space-between}.justify-center{-ms-flex-pack:center;justify-content:center}.button{cursor:pointer}table{border-collapse:collapse}td{border:var(--border-width,1px) solid var(--border-color,#e4e7e7);text-align:center;width:var(--cell-width,36px);height:var(--cell-height,36px)}td.borderless{border:none}td.day-name{font-size:12px}td.today{color:#9e9c9c}td.selected{background-color:var(--selected-bg-color,#00a699);color:#fff}td.in-range{background-color:var(--in-range-bg-color,#00a699);color:#fff}td.other-month{visibility:var(--other-month-visibility,hidden);border:var(--other-month-border,none)}td.day:not(.other-month):hover{background-color:var(--hover-bg-color,#dcdcdc);color:var(--hover-color,#000);cursor:pointer}"; },
        enumerable: true,
        configurable: true
    });
    return CalyCalendar;
}());
export { CalyCalendar as caly_calendar };
